﻿###############################################################################
# KoXo Tools
# 01/10/2022
###############################################################################
function banner {
    Param($MenuLines)
    $MarginWidth = 2
    $MarginChar = ""
    $MaxWidth = 0;
    $BorderTextColor = "Yellow"
    $BorderBackGroundColor = "DarkGray"
    $TextColor = "White"
    $BgColor = "DarkGray"
    ForEach ($Line In $MenuLines) {
        If ($Line.Length -gt $MaxWidth) { $MaxWidth = $Line.Length }
    }
    If ($MaxWidth -gt 0) {
        Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
        For ($i = 0; $i -lt $MaxWidth + 2 * $MarginWidth; $i++) {
            Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
        }
        Write-Host "╗" -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
        ForEach ($Line In $MenuLines) {
            Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
            Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth), (pad_both $Line $MaxWidth), $MarginChar.PadRight($MarginWidth)) -NoNewline -ForegroundColor $TextColor -BackgroundColor $BgColor
            Write-Host "║" -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
        }
        Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
        For ($i = 0; $i -lt $MaxWidth + 2 * $MarginWidth; $i++) {
            Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
        }
        Write-Host "╝" -ForegroundColor $BorderTextColor -BackgroundColor $BorderBackGroundColor
    }
}
###############################################################################
function pad_both {
    Param ([string] $Source, [int] $Length)
    $Spaces = $Length - $Source.Length
    $PadLeft = $Spaces / 2 + $Source.Length
    return $Source.PadLeft($PadLeft).PadRight($Length)
}
###############################################################################
function log_message {
    Param($Header, $Message)	
    $LogDate = Get-Date
    $LogInfo = "[$($LogDate)]: [$($Header)] $($Message)"
    $LogInfo | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_errors_count {
    $S = "TotalErrorsCount=$($global:total_errors_count)"
    $S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_errors_count {
    Param($Group)
    $S = "ErrorsCount[$($Group)]=$($global:errors_count)"
    $S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_warnings_count {
    $S = "TotalWarningsCount=$($global:total_warnings_count)"
    $S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_warnings_count {
    Param($ID)
    $S = "WarningsCount[$($ID)]=$($global:warnings_count)"
    $S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function check_version {
    $S = Get-Host
    if ($S.Version.Major -lt "2") {
        $S = "Powershell's version must be greater or equal to 2.0 !"
        log_message "Version" $S
        $global:total_errors_count++
        write-host $S -foregroundcolor "red"
        write-host "Press 'Enter' to continue ..."
        Read-Host
        exit
    }
}
###############################################################################
function set_proxy {
    Param($Enabled, $Url, $Port, $User, $Password)
    if ($Enabled) {
        $Cred = get_cred $User $Password
        $Proxy = -Join ("http://", $Url, ":", $Port)
        $Site = "https://provisioningapi.microsoftonline.com/provisioningwebservice.svc"
        Invoke-WebRequest -Uri $Site -Proxy $Proxy -ProxyCredential $Cred -ErrorAction SilentlyContinue
    }
}
###############################################################################
function get_cred {
    Param([string] $MyName, $MyPassword)
    $error.Clear()
    # Set credential
    if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  ) {
        $Cred = Get-Credential
    }
    else {
        $SecuredPassword = ConvertTo-SecureString $MyPassword -AsPlainText -Force
        $Cred = New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword 
    }
    return $Cred
} 
###############################################################################
function import_module {
    Param($Name)
    $Mod = Import-Module -Name $Name -PassThru -ErrorAction SilentlyContinue
    If ($Mod -ne $null) {
        $S = "Powershell module ""$($Mod.Name)"" V$($Mod.Version) loaded"
        Write-Host $S -Foregroundcolor "Green"
        Log_Message "OK" $S
        Return $true
    }
    else {
        $S = "Powershell module ""$($Name)"" not found !"
        Write-Host $S -Foregroundcolor "Red"
        Log_Message "ERROR" $S
        Return $false
    }
}
###############################################################################
function import_modules {
    Param([String]$Modules)
    $Res = $true
    $TabModules = $Modules.Split(",")
    ForEach ($Module In $TabModules) {
        $Temp = import_module $Module.Trim()
        if ($Temp -eq $false) { $Res = $false }
    }
    Return $Res
}
###############################################################################
function install_update_module {
    Param([string]$Name)
    $LocalModule = Get-InstalledModule -Name $Name -ErrorAction SilentlyContinue
    Write-Host "Checking for $($Name) module on Internet ..." -NoNewLine
    $OnlineModule = Find-Module -Name $Name -ErrorAction SilentlyContinue
    Write-Host
    If ($LocalModule -ne $null) {
        $S = "$($Name) V$($LocalModule.Version) module is already installed"
        Write-Host $S -ForegroundColor Cyan
        Log_Message "INFO" $S 
    }
    else {
        If ($OnlineModule -ne $null) {
            Write-Host "Installing module $($Name) V$($OnlineModule.Version) ..." -NoNewLine
            $Error.Clear()
            Install-Module -Name $Name -AllowClobber -Force -ErrorAction SilentlyContinue
            Write-Host
            # Passthru not available on all versions
            If ([String]::IsNullorEmpty($error[0])) {
                $InstalledModule = Get-InstalledModule -Name $Name -ErrorAction SilentlyContinue   
                $S = "Module $($Name) V$($InstalledModule.Version) is installed"
                Write-Host $S -ForegroundColor Green
                Log_Message "OK" $S
            }
            else {
                $global:total_errors_count++
                $S = "Module $($Name) not installed ! Error=$($error[0])"
                Write-Host $S -ForegroundColor Red
                Log_Message "ERROR" $S
            }
            Return 
        }
    }
    # Module can be updated ?
    If (($OnlineModule -ne $null) -and ($LocalModule -ne $null)) {
        $UpdateAvailable = compare_text_versions $LocalModule.Version $OnlineModule.Version
        If ($UpdateAvailable -eq -1) {
            Write-Host "Update module $($Name) from V$($LocalModule.Version) to V$($OnlineModule.Version) ? [Y/N] : " -NoNewline
            $Response = Read-Host
            If ($Response.ToLower() -eq "y") {
                Write-Host "Updating the $($Name) module from V$($LocalModule.Version) to V$($OnlineModule.Version) ..."
                $Error.Clear()
                Update-Module -Name $Name -Force -ErrorAction SilentlyContinue
                If ([String]::IsNullorEmpty($error[0])) {
                    $UpdatedModule = Get-InstalledModule -Name $Name -ErrorAction SilentlyContinue
                    $S = "Module $($Name) V$($UpdatedModule.Version) is updated to to V$($OnlineModule.Version)"
                    Write-Host $S -ForegroundColor Green
                    Log_Message "OK" $S
                }
                else {
                    $global:total_errors_count++
                    $S = "Module $($Name) cannot be updated ! Error=$($error[0])"
                    Write-Host $S -ForegroundColor Red
                    Log_Message "ERROR" $S
                }
            }
        }
    }
}
###############################################################################
function display_user {
    Param ($UserDisplayName, $UserEmail)
    Write-Host "User: """ -nonewline
    Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
    Write-Host """ Email: """ -nonewline
    Write-Host $UserEmail -foregroundcolor "Green" -nonewline
    Write-host """"	
}
###############################################################################
function get_os_version {
    Return (Get-WmiObject -class Win32_OperatingSystem).Version
}
###############################################################################
function get_os_name {
    Return (Get-WmiObject -class Win32_OperatingSystem).Caption
}
###############################################################################
# -1 : V1<V2  |  0 : V1=V2  |  1 : V1>V2   
function compare_text_versions {
    Param([string]$Version1, [string]$Version2)
    $V1 = [System.Version]$Version1
    $V2 = [System.Version]$Version2
    $Res = $V1.CompareTo($V2)
    Return $Res
}
###############################################################################
function get_wmf_version {
    $Major = [math]::max(0, $PSVersionTable.PSVersion.Major) 
    $Minor = [math]::max(0, $PSVersionTable.PSVersion.Minor)
    $Build = [math]::max(0, $PSVersionTable.PSVersion.Build)
    $Revision = [math]::max(0, $PSVersionTable.PSVersion.Revision)
    Return "$($Major).$($Minor).$($Build).$($Revision)"
}
###############################################################################
function Remove-StringDiacritics {
  param ([String]$sToModify = [String]::Empty)
  foreach ($s in $sToModify) # Param may be a string or a list of strings
  {
    if ($sToModify -eq $null) {return [string]::Empty}
    $sNormalized = $sToModify.Normalize("FormD")
    foreach ($c in [Char[]]$sNormalized)
    {
      $uCategory = [System.Globalization.CharUnicodeInfo]::GetUnicodeCategory($c)
      if ($uCategory -ne "NonSpacingMark") {$res += $c}
    }
    return $res
  }
}
###############################################################################
function Remove-SpecialCharacters {
    param ([String]$sourceStringToClean = [String]::Empty)
    return $sourceStringToClean -replace '[^a-zA-Z0-9.-_]', ''
}
